@@ -221,6 +221,15 @@ pub struct EnvPoint<T> {
     pub curve: CurveKind<T>,
 }
 
+/// Default bezier tangent values are all zeroes.
+/// This trait implements the constructors for those values.
+/// only I32Color needs to implement this manually, since volume (i32) and Position are already all zeroes in their default values.
+pub trait BezierDefault: Default {
+    fn bezier_default() -> Self {
+        Default::default()
+    }
+}
+
 #[derive(Debug, PartialEq, Copy, Clone, Serialize, Deserialize)]
 pub struct Position {
     pub x: i32,
@@ -237,7 +246,7 @@ pub struct I32Color {
     pub a: i32,
 }
 
-#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
+#[derive(Default, Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct Env<T> {
     pub name: String,
     pub synchronized: bool,
@@ -419,7 +428,7 @@ pub struct Quad {
     pub color_env_offset: i32,
 }
 
-#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
+#[derive(Default, Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct QuadsLayer {
     pub name: String,
     pub detail: bool,
@@ -452,7 +461,7 @@ pub struct SoundSource {
     pub shape: SoundShape,
 }
 
-#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
+#[derive(Default, Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct SoundsLayer {
     pub name: String,
     pub detail: bool,
