@@ -1,5 +1,6 @@
 use crate::map::*;
 use crate::compression::DecompressionError;
+use crate::constants;
 use crate::convert::{To, TryTo};
 
 use image::RgbaImage;
@@ -226,7 +227,7 @@ impl TwMap {
     }
 
     pub fn check_images(&self) -> Result<(), ImageError> {
-        Image::check_all(&self.images)
+        Image::check_all(&self.images, self.version)
     }
 
     pub fn check_envelopes(&self) -> Result<(), EnvelopeError> {
@@ -322,6 +323,8 @@ pub enum ImageErrorKind {
     NameLength(#[from] StringLenError),
     #[error("{0}")]
     Compressed(#[from] CompressedImageError),
+    #[error("'{0}' is not a valid external image in version {1:?}")]
+    InvalidExternalImage(String, Version),
     #[error("Width is too big: {0}, max is {}", i32::MAX)]
     TooBigWidth(u32),
     #[error("Height is too big: {0}, max is {}", i32::MAX)]
@@ -396,14 +399,14 @@ impl CheckData for CompressedData<RgbaImage, ImageLoadInfo> {
 }
 
 impl Image {
-    fn check_all(images: &[Image]) -> Result<(), ImageError> {
+    fn check_all(images: &[Image], version: Version) -> Result<(), ImageError> {
         check_amount(images)
             .map_err(|err| ImageError {
                 index: None,
                 kind: err.into(),
             })?;
         for (i, image) in images.iter().enumerate() {
-            image.check()
+            image.check(version)
                 .map_err(|kind| ImageError {
                     index: Some(i),
                     kind
@@ -412,12 +415,17 @@ impl Image {
         Ok(())
     }
 
-    fn check(&self) -> Result<(), ImageErrorKind> {
+    fn check(&self, version: Version) -> Result<(), ImageErrorKind> {
         check_string_sanitization(self.name(), "png")?;
         check_string_length(self.name(), Image::MAX_NAME_LENGTH)?;
 
-        if let Image::Embedded(image) = self {
-            image.image.check_data()?;
+        match self {
+            Image::Embedded(emb) => emb.image.check_data()?,
+            Image::External(ex) => {
+                if !constants::is_external_name(&ex.name, version) {
+                    return Err(ImageErrorKind::InvalidExternalImage(ex.name.clone(), version));
+                }
+            }
         }
         Ok(())
     }
@@ -451,7 +459,9 @@ pub enum EnvPointErrorKind {
     #[error("Curve type with id {0} is unknown")]
     UnknownCurveType(i32),
     #[error("Timestamp is at {1} while the one before was at {0}")]
-    Ordering(i32, i32)
+    Ordering(i32, i32),
+    #[error("Timestamp is negative, must be >= 0")]
+    NegativeTimeStamp(i32),
 }
 
 impl fmt::Display for EnvPointError {
@@ -475,6 +485,15 @@ impl fmt::Display for EnvelopeError {
 
 fn check_env_points<T>(points: &[EnvPoint<T>]) -> Result<(), EnvPointError> {
     use EnvPointErrorKind::*;
+    if let Some(point) = points.first() {
+        if point.time < 0 {
+            return Err(EnvPointError {
+                kind: NegativeTimeStamp(point.time),
+                index: 0
+            })
+        }
+    }
+
     for (i, pair) in points.windows(2).enumerate() {
         if pair[0].time > pair[1].time {
             Err(EnvPointError {
