@@ -1,8 +1,5 @@
 use crate::map::*;
-use crate::constants::{
-    vanilla_opaque_tables::VANILLA_EXTERNAL_IMAGES,
-    ddnet_opaque_tables::DDNET_EXTERNAL_IMAGES
-};
+use crate::constants;
 use crate::map_checks::PhysicsLayerChecking;
 use crate::convert::{To, TryTo};
 
@@ -236,7 +233,7 @@ impl TwMap {
     pub fn remove_unused_groups(&mut self) -> u16 {
         let mut removed = 0;
         let mut i = 0;
-        while i > self.groups.len() {
+        while i < self.groups.len() {
             if self.groups[i].layers.is_empty() {
                 removed += 1;
                 self.groups.remove(i);
@@ -464,15 +461,7 @@ impl Image {
     pub fn calc_opaque_table(&self, version: Version) -> [[bool; 16]; 16] {
         match self {
             Image::External(image) => {
-                let opaque_tables = match version {
-                    Version::Teeworlds07 => VANILLA_EXTERNAL_IMAGES.to_vec(),
-                    Version::DDNet06 => DDNET_EXTERNAL_IMAGES.to_vec(),
-                };
-                match opaque_tables.into_iter()
-                    .find(|table| table.0 == image.name) {
-                    None => [[false; 16]; 16],
-                    Some(opaque_table) => opaque_table.1,
-                }
+                constants::external_opaque_table(&image.name, version).unwrap_or([[false; 16]; 16])
             }
             Image::Embedded(image) => calc_opaque_table(image.image.unwrap_ref())
         }
@@ -488,9 +477,16 @@ impl TwMap {
         for group in &mut self.groups {
             for layer in &mut group.layers {
                 if let Layer::Tiles(layer) = layer {
-                    let opaque_table = match layer.image {
-                        None => &[[false; 16]; 16],
-                        Some(index) => &tables[index.to::<usize>()],
+                    let opaque_table = {
+                        if layer.color.a != 255 {
+                            &[[false; 16]; 16]
+                        }
+                        else {
+                            match layer.image {
+                                None => &[[false; 16]; 16],
+                                Some(index) => &tables[index.to::<usize>()],
+                            }
+                        }
                     };
                     let tiles = layer.tiles.unwrap_mut();
                     for tile in tiles {
@@ -501,121 +497,86 @@ impl TwMap {
             }
         }
     }
-}
 
-pub trait ZeroUnusedTileParts: AnyTile {
-    /// Unused as in 'not editable by common tools'
-    fn zero_unused_tile_parts(&mut self) -> usize { 0 }
+    /// Set the width and height of external images to their default values.
+    pub fn set_external_image_dimensions(&mut self) {
+        for image in &mut self.images {
+            if let Image::External(ex) = image {
+                if let Some((width, height)) = constants::external_dimensions(&ex.name, self.version) {
+                    ex.width = width;
+                    ex.height = height;
+                }
+            }
+        }
+    }
 }
 
 impl TileFlags {
     fn clear_unused(&mut self) -> bool {
         let cleared = *self & TileFlags::all();
-        if cleared != * self {
-            *self = cleared;
-            true
-        }
-        else {
-            false
-        }
+        let changed = cleared != *self;
+        *self = cleared;
+        changed
     }
 
     fn clear_unused_and_opaque(&mut self) -> bool {
         let cleared = *self & (TileFlags::all() - TileFlags::OPAQUE);
-        if cleared != * self {
-            *self = cleared;
-            true
-        }
-        else {
-            false
-        }
-    }
-}
-
-impl ZeroUnusedTileParts for Tile {
-    fn zero_unused_tile_parts(&mut self) -> usize {
-        let mut changed = 0;
-        if self.skip != 0 {
-            changed += 1;
-            self.skip = 0;
-        }
-        if self.unused != 0 {
-            changed += 1;
-            self.unused = 0;
-        }
-        if self.flags.clear_unused() {
-            changed += 1;
-        }
+        let changed = cleared != *self;
+        *self = cleared;
         changed
     }
 }
 
-impl ZeroUnusedTileParts for GameTile {
-    fn zero_unused_tile_parts(&mut self) -> usize {
-        let mut changed = 0;
-        if self.skip != 0 {
-            changed += 1;
-            self.skip = 0;
-        }
-        if self.unused != 0 {
-            changed += 1;
-            self.unused = 0;
-        }
-        if self.flags.clear_unused_and_opaque() {
-            changed += 1;
-        }
+impl Tile {
+    fn zero_unused_tile_parts(&mut self) -> bool {
+        let changed = self.skip != 0 || self.unused != 0 || self.flags.clear_unused();
+        self.skip = 0;
+        self.unused = 0;
+        changed // note that the flags are already cleared in the first line
+    }
+}
+
+impl GameTile {
+    fn zero_unused_tile_parts(&mut self) -> bool {
+        let changed = self.skip != 0 || self.unused != 0 || self.flags.clear_unused_and_opaque();
+        self.skip = 0;
+        self.unused = 0;
+        changed // note that the flags are already cleared in the first line
+    }
+}
+
+impl Speedup {
+    fn zero_unused_tile_parts(&mut self) -> bool {
+        let changed = self.unused_padding != 0;
+        self.unused_padding = 0;
         changed
     }
 }
 
-impl ZeroUnusedTileParts for Speedup {
-    fn zero_unused_tile_parts(&mut self) -> usize {
-        if self.unused_padding != 0 {
-            self.unused_padding = 0;
-            1
-        }
-        else {
-            0
-        }
+impl Switch {
+    fn zero_unused_tile_parts(&mut self) -> bool {
+        self.flags.clear_unused_and_opaque()
     }
 }
 
-impl ZeroUnusedTileParts for Tele {}
-
-impl ZeroUnusedTileParts for Switch {
-    fn zero_unused_tile_parts(&mut self) -> usize {
-        let mut changed = 0;
-        if self.flags.clear_unused_and_opaque() {
-            changed += 1;
-        }
-        changed
-    }
-}
-
-impl ZeroUnusedTileParts for Tune {}
-
-fn zero_unused_tile_parts<T: ZeroUnusedTileParts>(tiles: &mut Array2<T>) -> usize {
-    let mut changed = 0;
-    tiles.iter_mut()
-        .for_each(|tile| changed += tile.zero_unused_tile_parts());
-    changed
-}
-
 impl TwMap {
-    pub fn zero_unused_tile_parts(&mut self) -> usize {
+    pub fn zero_unused_tile_parts(&mut self) -> u64 {
         let mut changed = 0;
         for group in &mut self.groups {
             for layer in &mut group.layers {
                 use Layer::*;
                 changed += match layer {
-                    Game(l) => zero_unused_tile_parts(l.tiles.unwrap_mut()),
-                    Tiles(l) => zero_unused_tile_parts(l.tiles.unwrap_mut()),
-                    Front(l) => zero_unused_tile_parts(l.tiles.unwrap_mut()),
-                    Tele(l) => zero_unused_tile_parts(l.tiles.unwrap_mut()),
-                    Speedup(l) => zero_unused_tile_parts(l.tiles.unwrap_mut()),
-                    Switch(l) => zero_unused_tile_parts(l.tiles.unwrap_mut()),
-                    Tune(l) => zero_unused_tile_parts(l.tiles.unwrap_mut()),
-                    Sounds(_) | Invalid(_) | Quads(_) => 0
+                    Game(l) => l.tiles.unwrap_mut()
+                        .map_mut(|t| t.zero_unused_tile_parts().to::<u64>()).sum(),
+                    Tiles(l) => l.tiles.unwrap_mut()
+                        .map_mut(|t| t.zero_unused_tile_parts().to::<u64>()).sum(),
+                    Front(l) => l.tiles.unwrap_mut()
+                        .map_mut(|t| t.zero_unused_tile_parts().to::<u64>()).sum(),
+                    Speedup(l) => l.tiles.unwrap_mut()
+                        .map_mut(|t| t.zero_unused_tile_parts().to::<u64>()).sum(),
+                    Switch(l) => l.tiles.unwrap_mut()
+                        .map_mut(|t| t.zero_unused_tile_parts().to::<u64>()).sum(),
+                    Tele(_) | Tune(_) | Sounds(_) | Invalid(_) | Quads(_) => 0
                 }
             }
         }
@@ -1389,12 +1350,12 @@ impl Envelope {
 }
 
 impl TwMap {
-    // move cosmetic layers from the game group in a group before/after the game group, depending on position relative to the game layer
-    fn isolate_physics_layers(&mut self) {
+    /// Move cosmetic layers from the physics group into separate groups, keeping the render order
+    pub fn isolate_physics_layers(&mut self) {
         let index = self.groups.iter().position(|g| g.is_physics_group()).unwrap();
         let game_group = &mut self.groups[index];
-        let mut front_group = Group { name: String::from("GameFront"), ..Group::default() };
-        let mut back_group = Group { name: String::from("GameBack"), ..Group::default() };
+        let mut front_group = Group { name: String::from("v Front"), ..Group::physics() };
+        let mut back_group = Group { name: String::from("^ Back"), ..Group::physics() };
         let mut i = 0;
         let mut after = false;
         while i < game_group.layers.len() {
