@@ -333,11 +333,9 @@ impl Default for Group {
 }
 
 impl Group {
-    /// Constructor for the game group, make sure not to change any values apart from the layers
-    pub fn game() -> Self {
-        let mut group = Group::default();
-        group.name = String::from("Game");
-        group
+    /// Constructor for the physics group, make sure not to change any values apart from the layers
+    pub fn physics() -> Self {
+        Group { name: String::from("Game"), ..Group::default()}
     }
 }
 
