@@ -53,6 +53,22 @@ fn editor_test() {
     map_load_save_load_save_eq("tests/editor.map", Version::DDNet06);
 }
 
+#[test]
+fn tile_flags_teeworlds_test() {
+    let mut original_map = TwMap::parse_file("tests/tileflags_teeworlds.map").unwrap();
+    assert_eq!(original_map.version, Version::Teeworlds07);
+    original_map.load().unwrap();
+    let mut own_map = original_map.clone();
+
+    own_map.process_tile_flag_opaque();
+    own_map.set_external_image_dimensions();
+    assert_eq!(own_map, original_map);
+    own_map.version = Version::DDNet06;
+    own_map.process_tile_flag_opaque();
+    own_map.set_external_image_dimensions();
+    assert_ne!(own_map, original_map);
+}
+
 #[test]
 fn tile_flags_ddnet_test() {
     let mut original_map = TwMap::parse_file("tests/tileflags_ddnet.map").unwrap();
@@ -61,9 +77,11 @@ fn tile_flags_ddnet_test() {
     let mut own_map = original_map.clone();
 
     own_map.process_tile_flag_opaque();
+    own_map.set_external_image_dimensions();
     assert_eq!(own_map, original_map);
     own_map.version = Version::Teeworlds07;
     own_map.process_tile_flag_opaque();
+    own_map.set_external_image_dimensions();
     assert_ne!(own_map, original_map);
 }
 
